@@ -2,6 +2,7 @@ import requests
 import json
 import time
 import platform
+import itertools
 
 
 class Query(object):
@@ -12,21 +13,30 @@ class Query(object):
 
     _BASE_URL = 'https://data.thinknum.com'
 
-    def __init__(self, client_id, client_secret, proxies={}, verify=True):
+    def __init__(self, client_id, client_secret, proxies={}, verify=True, timeout=180):
+        if not client_id or not isinstance(client_id, str):
+            raise Exception('Invalid parameter: client_id')
+        if not client_secret or not isinstance(client_secret, str):
+            raise Exception('Invalid parameter: client_secret')
+        if not isinstance(proxies, dict):
+            raise Exception('Invalid parameter: proxies')
+        if not isinstance(verify, bool):
+            raise Exception('Invalid parameter: verify')
+        if not isinstance(timeout, (int, type(None))):
+            raise Exception('Invalid parameter: timeout')
+
         self._client_id = client_id
         self._client_secret = client_secret
         self._proxies = proxies
         self._verify = verify
-
         self._token = None
         self._authenticate()
         self._headers = {
             "Accept": "application/json",
             "Authorization": "token {token}".format(token=self._token),
             "X-API-Version": "20151130",
-            "User-Agent": "Python API 1.82 / {local_version}".format(local_version=platform.python_version())
+            "User-Agent": "Python API 1.84 / {local_version}".format(local_version=platform.python_version())
         }
-
         self._dataset_id = None
         self._tickers = []
         self._filters = []
@@ -36,6 +46,7 @@ class Query(object):
         self._sorts = []
         self._start = 1
         self._limit = 100000
+        self._timeout = timeout
 
     def _raise_exception_from_response(self, response):
         try:
@@ -237,14 +248,14 @@ class Query(object):
     def get_data(self, dataset_id, start=None, limit=None):
         if not dataset_id:
             return {}
-
-        if start and limit:
-            self._start = start
-            self._limit = limit
         if self._start < 1:
             raise Exception('Start offset is less than 1')
         if self._limit < 1:
             raise Exception('Limit offset is less than 1')
+        if start:
+            self._start = start
+        if limit:
+            self._limit = limit
 
         # 1. POST
         response = self._requests(
@@ -263,7 +274,7 @@ class Query(object):
 
         # 2. HEAD
         is_created = False
-        for index in range(60):
+        for index in range(self._timeout) if self._timeout else itertools.count():
             response = self._requests(
                 method='head',
                 url='{base_url}/datasets/{dataset_id}/query/{query_id}'.format(
